-------------------------------- MODULE gcd --------------------------------

EXTENDS Integers, TLAPS

Number == Nat \ {0}

Divides(p, n) == \E q \in Int : n = q * p

DivisorsOf(n) == {p \in Int : Divides(p, n)}

SetMax(S) == CHOOSE i \in S : \A j \in S : i >= j

GCD(m, n) ==
   SetMax(DivisorsOf(m) \cap DivisorsOf(n))

LEMMA LemmaDiv == \A m,n \in Nat \ {0} : 
             \E d \in Nat \ {0} : 
             Divides(d, m) /\ Divides(d, n) <=> Divides(d, m + n) /\ Divides(d, m - n)
  BY DEF Divides

(*
<1> SUFFICES ASSUME NEW m \in Number, 
                    NEW n \in Number, 
                    NEW d \in Int,
                    Divides(d, m),
                    Divides(d, n)
    PROVE Divides(d, m) ^ Divides(d, n) => Divides(d, m+n)            
<1>1. PICK q \in Number : m = q * d 
  BY DEF Divides
<1> QED
*)

THEOREM GCD1 == \A m \in Nat \ {0} : GCD(m, m) = m 
  <1> SUFFICES ASSUME NEW m \in Nat \ {0}
               PROVE  GCD(m, m) = m
    OBVIOUS
  <1>1. Divides(m, m)
    BY DEF Divides
  <1>2. \A i \in Nat : Divides(i, m) => (i <= m)
    BY DEF Divides
  <1> QED
    BY <1>1, <1>2 DEF GCD, SetMax, DivisorsOf, Divides
  
THEOREM GCD2 == \A m, n \in Nat \ {0} : GCD(m, n) = GCD(n, m) 
    BY DEF GCD, SetMax, DivisorsOf, Divides

THEOREM GCD3 == \A m, n \in Number : (n > m) => (GCD(m, n) = GCD(m, n-m)) 
  <1> SUFFICES ASSUME NEW m \in Number, NEW n \in Number,
                      n > m
               PROVE  GCD(m, n) = GCD(m, n-m)
    OBVIOUS 
(*  <1>1. Divides(d, n - m)
    <2> SUFFICES ASSUME NEW q \in Int, m = q * d, 
                        NEW r \in Int, n = r * d,
                        NEW s \in Int
                 PROVE Divides(d, n - m)             
      BY DEF Divides 
    <2>1. n - m = (r - q) * d
      BY DEF Divides
    <2> QED
      BY <2>1 DEF Divides
  <1>2. Divides(d, n)
    <3> SUFFICES ASSUME NEW q \in Nat \ {0},     m = q * d,
                        NEW r \in Nat \ {0}, n - m = r * d
                 PROVE Divides(d, n)
      BY DEF Divides
    <3>1. n = (q + r) * d
      BY DEF Divides
    <3> QED
      BY DEF Divides
  <1> QED
*)  
  <1> \A i \in Int : Divides(i, m) /\ Divides(i, n) 
                      <=> Divides(i, m) /\ Divides(i, n - m)
    BY DEF Divides, Number
  <1> QED
    BY DEF GCD, SetMax, DivisorsOf

   
=============================================================================
\* Modification History
\* Last modified Mon Nov 10 10:41:34 PST 2014 by Chris.Nott
\* Created Thu Nov 06 12:18:54 PST 2014 by Chris.Nott
