--------------------------- MODULE euclid_pluscal ---------------------------
EXTENDS Integers, gcd, TLAPS
CONSTANT M, N
ASSUME MNPosInt == 
       /\ M \in Nat \ {0}
       /\ N \in Nat \ {0}
       
(***************************************************************************
--algorithm Euclid {
  variables x = M, y = N;
  {
    while (x # y) {
      if (x < y) {y := y - x}
      else       {x := x - y}
    }         
  }
}
***************************************************************************)       
\* BEGIN TRANSLATION
VARIABLES x, y, pc

vars == << x, y, pc >>

Init == (* Global variables *)
        /\ x = M
        /\ y = N
        /\ pc = "Lbl_1"

Lbl_1 == /\ pc = "Lbl_1"
         /\ IF x # y
               THEN /\ IF x < y
                          THEN /\ y' = y - x
                               /\ x' = x
                          ELSE /\ x' = x - y
                               /\ y' = y
                    /\ pc' = "Lbl_1"
               ELSE /\ pc' = "Done"
                    /\ UNCHANGED << x, y >>

Next == Lbl_1
           \/ (* Disjunct to prevent deadlock on termination *)
              (pc = "Done" /\ UNCHANGED vars)

Spec == Init /\ [][Next]_vars

Termination == <>(pc = "Done")

\* END TRANSLATION

PartialCorrectness == pc = "Done" => (x = y) /\ (x = GCD(M, N))

TypeOK == /\ x \in Nat \ {0}
          /\ y \in Nat \ {0}
          
Inv == /\ TypeOK
       /\ GCD(x, y) = GCD(M, N)
       /\ pc = "Done" => (x = y)
       
THEOREM Spec => []PartialCorrectness 
<1>. USE DEF Inv, TypeOK, Init, PartialCorrectness
<1>1. Init => Inv  BY MNPosInt
<1>2. Inv /\ [Next]_vars => Inv'  BY GCD2, GCD3 DEF Next, Lbl_1, vars
<1>3. Inv => PartialCorrectness  BY GCD1
<1>4. QED  BY <1>1, <1>2, <1>3, PTL DEF Spec


(*
<1>1. Init => Inv
  BY MNPosInt DEF Init, Inv, TypeOK 
(*
<1> Inv /\ [Next]_vars => Inv'
  <2> SUFFICES ASSUME Inv,
                      [Next]_vars
               PROVE  Inv'
    OBVIOUS
  <2>1. ASSUME abc
        PROVE  Inv'
  <2>2. ASSUME pc = "Done" /\ UNCHANGED vars
        PROVE  Inv'
  <2>3. ASSUME UNCHANGED vars
        PROVE  Inv'
  <2>4. QED
    BY <2>1, <2>2, <2>3 DEF Next
*)

<1>2. Inv /\ [Next]_vars => Inv'
(*  BY MNPosInt, GCD2, GCD3 DEF Inv, TypeOK, Next, abc, vars*)
  <2> SUFFICES ASSUME Inv,
                      [Next]_vars
                PROVE  Inv'
    OBVIOUS
  <2>1. CASE abc
    BY MNPosInt, GCD2, GCD3, abc DEF Inv, TypeOK, Next, abc, vars

  <2>2. ASSUME pc = "Done" /\ UNCHANGED vars
        PROVE  Inv'
    BY Inv, <2>2 DEF Inv, TypeOK, vars
  <2>3. ASSUME UNCHANGED vars
        PROVE  Inv'
    BY Inv, <2>3 DEF Inv, TypeOK, vars
  <2>4. QED
    BY <2>1, <2>2, <2>3 DEF Next

<1>3. Inv => PartialCorrectness
  BY GCD1 DEF Inv, TypeOK, Init, PartialCorrectness
<1> QED
  BY <1>1, <1>2, <1>3, PTL DEF Spec
*)

=============================================================================
\* Modification History
\* Last modified Mon Nov 10 11:00:48 PST 2014 by Chris.Nott
\* Created Thu Nov 06 12:15:37 PST 2014 by Chris.Nott
